-- GLOBAL --
ATT = select(2, ...); 
-- MODULE --
local _G = getfenv(0);
setmetatable(ATT, {__index = _G});
setfenv(1, ATT);
-- LOCAL --
title		= "Alt-Tab Toggle"
version		= "1.0.1"
titleFull	= title.." version "..version

coreFrame = CreateFrame("Frame");
core	= LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceEvent-3.0") -- 
local L = LibStub("AceLocale-3.0"):GetLocale(title, true)
local P; --db.profile

ATTFrame = CreateFrame("Frame",nil,UIParent)
lastOnUpdate = 0;
ATTActive = false;
directory 	= "Interface\\Addons\\"..title.."\\";
timers = {};

--Globals
local GetTime_				= _G.GetTime;
local InCombatLockdown_		= _G.InCombatLockdown;
local PlaySoundFile_		= _G.PlaySoundFile;
local table_getn			= _G.table.getn;
local print_				= _G.print;
local select_				= _G.select;
local UnitDebuff_			= _G.UnitDebuff;
local GetBattlefieldStatus_	= _G.GetBattlefieldStatus;
local GetRealZoneText_		= _G.GetRealZoneText;
local ConsoleExec_			= _G.ConsoleExec;
local SetCVar_				= _G.SetCVar;
local GetCVar_				= _G.GetCVar;

-- Default Settings
defaultSettings = {profile = {}}
defaultSettings.profile.attAlertSound		= true;
defaultSettings.profile.soundOnly			= false;
defaultSettings.profile.ATTDeserter			= true;
defaultSettings.profile.ATTIdle				= true;
defaultSettings.profile.attAFKActivate		= false;
defaultSettings.profile.ATTWhisper			= true;
defaultSettings.profile.ATTCombat			= true;
defaultSettings.profile.ATTFlight			= true;
defaultSettings.profile.ATTReadyCheck		= true;
defaultSettings.profile.ATTSummon			= true;
defaultSettings.profile.ATTInvite			= true;
defaultSettings.profile.ATTBGPop			= true;
defaultSettings.profile.ATTEnemyDetected	= false;
defaultSettings.profile.attAltTabDetection	= true;
defaultSettings.profile.ATTBGGatesOpening	= true;

-- Echo Strings
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.


local strATTReason 		= "Alt-Tab Toggle: |cffffff00%s|r"-- 
local strATTActivated 	= "Alt-Tab Toggle: |cff00ff00Activated|r"
local strATTDeactivated	= "Alt-Tab Toggle: |cffff0000Deactivated|r"
local strNoSoundsInBackground = "'Sounds in Background' is disabled, Enable it if you want sounds when alt tabbed.";
local strATTIsDisabled = "ATT is disabled, enable it on the options screen first."

--Internal settings
local atDetectionWait	= 5;-- if our OnUpdate function hasn't updated in 5 seconds, assume we've alt tabbed.
ignoreEnemyZones = {
--~ 	["Shattrath City"]	= true,
--~ 	["Dalaran"] 		= true, --unitispvp
--~ 	[""]				= true,--When we login, the GetZone API doesn't return the zone name.
}

-- Ace3 Functions
function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("ATT_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")

	P = db.profile;

	core:RegisterChatCommand("att", "MySlashProcessorFunc")

	local config = LibStub("AceConfig-3.0");
	local dialog = LibStub("AceConfigDialog-3.0");
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, "ATT v"..version)
end


----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
--~ 	InterfaceOptionsFrame_OpenToCategory(coreOpts)
	if core:IsEnabled() then
		ATTToggleMode(nil, nil, "Slash command");
	else
		print_(strATTIsDisabled)
	end
end

local regEvents = {
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_WHISPER",
	"PLAYER_REGEN_DISABLED",
	"PLAYER_CONTROL_GAINED",
	"PLAYER_ENTERING_WORLD",
	"READY_CHECK",
	"CONFIRM_SUMMON",
	"PARTY_INVITE_REQUEST",
	"UPDATE_BATTLEFIELD_STATUS",
	"UNIT_AURA",
	"COMBAT_LOG_EVENT_UNFILTERED",
	"CHAT_MSG_BG_SYSTEM_NEUTRAL",
}

function core:OnEnable()
	ATTFrame:SetScript("OnUpdate", function(this, duration)
		lastOnUpdate = GetTime_();
	end)

	for i, event in pairs(regEvents) do 
		self:RegisterEvent(event);
	end
	
	if GetCVar_("Sound_EnableSoundWhenGameIsInBG") == 0 then
		print_(strNoSoundsInBackground)
	end
	
	for name, data in pairs(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = false;
		end
	end
end

function core:OnDisable()
	ATTFrame:SetScript("OnUpdate", nil)
	
	for name, data in pairs(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = true;
		end
	end
	ATTToggleMode(false, false, "Addon Disabled")
end



----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

function IsAltTabbed()
	if InCombatLockdown_() then
		--we're in combat, probably not alt tabbed. note this doesn't return true when we enter combat.
		return false;
	end
	if GetCVar_("gxWindow") == 0 then
		if GetTime_() - lastOnUpdate > atDetectionWait then
			return true;
		end	
	end
	return false;
end


function core:CHAT_MSG_BG_SYSTEM_NEUTRAL(event, ...)
	local message  = ...-- ***
	message = message:lower();
	if	message:find("the battle for arathi basin has begun!") or --AB
		message:find("the battle for alterac valley has begun") or --AV
		message:find("let the battle for warsong gulch begin!") or --WSG
		message:find("the battle has begun") then --EotS

		if P.ATTBGGatesOpening == true then
			FocusWoW("Gates opened");
		end

	end
end

function core:CONFIRM_SUMMON(event, ...)
	if P.ATTSummon == true then
		FocusWoW("Summon");
	end
end

function core:READY_CHECK(event, ...)
	if P.ATTReadyCheck == true then
		FocusWoW("Ready Check");
	end
end


function core:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	if ignoreEnemyZones[GetRealZoneText_()] then
		return;
	end

	if P.ATTEnemyDetected == true and not InBattleground() then
		local timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags  = select_(1, ...)-- ***

		if (srcFlags) and FlagIsEnemy(srcFlags) then
			FocusWoW("Enemy detected ("..srcName..")");
		elseif FlagIsPlayer(dstFlags) and FlagIsEnemy(dstFlags) then
			FocusWoW("Enemy detected ("..dstName..")");
		end
	end
end

function FlagIsEnemy(flags)
	if bit.band(flags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE then
		return true;
	end
	return nil;
end

function FlagIsPlayer(flags)
	if bit.band(flags, COMBATLOG_OBJECT_TYPE_PLAYER) == COMBATLOG_OBJECT_TYPE_PLAYER then
		return true;
	end
	return nil;
end

function core:PLAYER_ENTERING_WORLD(event, ...)
	lastFocusTime = GetTime_();--if we exit a BG PLAYER_CONTROL_GAINED will fire, we don't want that.
end

function core:PLAYER_REGEN_DISABLED(event, ...)
	if P.ATTCombat == true then
		FocusWoW("Entered combat");
	end
end

function core:CHAT_MSG_WHISPER(event, ...)
	if P.ATTWhisper == true then
		FocusWoW("Received whisper");
	end
end

function core:PARTY_INVITE_REQUEST(event, ...)
	if P.ATTInvite == true then
		FocusWoW("Invite");
	end
end

function core:PLAYER_CONTROL_GAINED(event, ...)
	if P.ATTFlight == true then
		FocusWoW("Flight land");
	end
end

function core:UPDATE_BATTLEFIELD_STATUS(event, ...)
	if P.ATTBGPop == true then
		local status, mapName, instanceID;
		for i=1, MAX_BATTLEFIELD_QUEUES do
			status, mapName, instanceID = GetBattlefieldStatus_(i)
			if status == "confirm" then
				FocusWoW("Battleground confirm");
			end
		end
	end
end

function core:CHAT_MSG_SYSTEM(event, ...)
	if P.attAFKActivate == true then
		local message, format  = ...;
		if message:find("You are now AFK") then --You are now AFK: Away from Keyboard

			ATTToggleMode(true, nil, "User went AFK")
			return;
		elseif message:find("You are no longer AFK.") then
			ATTToggleMode(false, nil, "User nolonger AFK")
			return;
		end
	end
end


isDeserter = false;
function core:UNIT_AURA(event, ...)
	if P.ATTDeserter == true then
		local unitID  = ...
		if unitID == "player" then
			local buffOn = IsDebuffOn("Deserter", unitID);
			if isDeserter == false then
				if buffOn then
					isDeserter = true;
				end
			else
				if not buffOn then
					FocusWoW("Deserter expirered");
					isDeserter = false;
				end
			end
		end
	end
	if P.ATTIdle == true then
		local unitID  = ...
		if unitID == "player" then
			if IsDebuffOn("Idle", unitID) then
				FocusWoW("Idle debuff gained");
			elseif IsDebuffOn("Inactive", unitID) then
				FocusWoW("Inactive debuff gained");
			end
		end
	end
end


function IsDebuffOn(spellName, unitID)
	local name;
	for i = 1, 40 do
		name  =  UnitDebuff_(unitID, i)
		if not name then break; end
		if name == spellName then
			return true;
		end
	end
	return false;
end

------------------------------
function Debug(from, ...)	--	/script DA.Debug("hey","bye")
-- simple print function.	--
------------------------------
	local tbl  = {...}
	local msg = tostring(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring(tbl[i]);
	end
	print_(strDebugFrom:format(from).." "..msg)
end

function FocusWoW(reason)
--~ 	Debug("FocusWoW", reason)
	if (P.attAltTabDetection == true and IsAltTabbed()) or ATTActive == true then --
		
		HardFocusWoW(reason, true)
		ATTActive = false;
	end
end


lastFocusTime = 0;
function HardFocusWoW(reason, force)--	/script V.HardFocusWoW()
	if GetTime_() - lastFocusTime < 5 then
		return;
	end
	if ATTActive == true or force == true then
		--I forget why this windowmode code is here, /shrug
--~ 		local WindowMode = GetCVar_("gxWindow");	--1 = windowmode, 0 = fullscreen.
--~ 		if (WindowMode == 1) then
--~ 			SetCVar_("gxWindow", "1"); --window mode
--~ 		else
--~ 			SetCVar_("gxWindow", "0"); --window full
--~ 		end
			
		if P.attAlertSound == true then -- and P.ATTDelayFocus == false
			TurnSoundsOn()
			PlaySoundFile_(directory.."Sounds\\ATTalert.mp3")
			core:ScheduleTimer("TurnSoundsOff", 2)--turn sounds off in 2 seconds.
		end
		
		if P.soundOnly == false then
			ConsoleExec_("gxRestart"); -- /console gxRestart
		end
		
		lastFocusTime = GetTime_();
		if reason then
			ATTToggleMode(false, true)
			print_(strATTReason:format(reason))
		else
			ATTToggleMode(false)
		end

	end
end



local soundSettings = {};
function TurnSoundsOn()
	--Sound_EnableSoundWhenGameIsInBG
	soundSettings.ALL = GetCVar_("Sound_EnableAllSound")
	soundSettings.ALLVol = GetCVar_("Sound_MasterVolume")
	soundSettings.SFX = GetCVar_("Sound_EnableSFX")
	soundSettings.SFXVol = GetCVar_("Sound_SFXVolume")
	soundSettings.BG = GetCVar_("Sound_EnableSoundWhenGameIsInBG")

	SetCVar_("Sound_EnableAllSound", 1)
	SetCVar_("Sound_MasterVolume", 1)
	SetCVar_("Sound_EnableSFX", 1)
	SetCVar_("Sound_SFXVolume", 1)
	SetCVar_("Sound_EnableSoundWhenGameIsInBG", 1)--this can't be set while alt tabbed? WTF!
end

function core:TurnSoundsOff()
	if soundSettings.ALL then
		SetCVar_("Sound_EnableAllSound", soundSettings.ALL)
	end
	if soundSettings.ALLVol then
		SetCVar_("Sound_MasterVolume", soundSettings.ALLVol)
	end
	if soundSettings.SFX then
		SetCVar_("Sound_EnableSFX", soundSettings.SFX)
	end
	if soundSettings.SFXVol then
		SetCVar_("Sound_SFXVolume", soundSettings.SFXVol)
	end
	if soundSettings.BG then
		SetCVar_("Sound_EnableSoundWhenGameIsInBG", soundSettings.BG)
	end
end

function ATTToggleMode(mode, silent, reason)
	if mode == nil then mode = not ATTActive; end
	if ATTActive ~= mode then
		ATTActive = mode;
		if (ATTActive == true) then
			if not silent then
				print_(strATTActivated, "||", reason)
			end
		else
			if not silent then
				print_(strATTDeactivated, "||", reason)
			end
		end
	end
end


function InBattleground()
	local iActive, iType = IsInInstance()
	if iActive and iType == "pvp" then
		--We're in a battlefield. Ofcourse there's enemies around.
		return true;
	end
	return false;
end

----------------
--     UI     --
----------------
CoreOptionsTable = {
	name = titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = L["Options"],
			type = "group",
			order = 1,
			args={}
		},
	},
}

CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= L["Enable"],
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable();
		else
			core:Disable();
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.attAFKActivate = {
	name = "AFK Activate",
	desc = "When you go away from the keyboard (/afk), ATT will activate then you're nolonger afk.",
	type = "toggle",
	order	= 2,
	set = function(info,val) 
		P.attAFKActivate = val;
	end,
	get = function(info) return P.attAFKActivate end
}

CoreOptionsTable.args.core.args.attAlertSound = {
	type = "toggle",	order	= 3,
	name = "Alert Sound",
	desc = "Play a sound when toggling",
	set = function(info,val) 
		P.attAlertSound = val;
	end,
	get = function(info) return P.attAlertSound end
}

CoreOptionsTable.args.core.args.soundOnly = {
	name = "Sound Only",
	desc = "Only play sound on toggle event, don't force WoW to forground.",
	type = "toggle",
	order	= 4,
	set = function(info,val) 
		P.soundOnly = val;
	end,
	get = function(info) return P.soundOnly end
}


CoreOptionsTable.args.core.args.attAltTabDetection = {
	name = "Alt Tab Detection",
	desc = "If your FPS hasen't refreshed in 5 seconds we assume you've alt tabbed.",
	type = "toggle",
	order	= 5,
	set = function(info,val) 
		P.attAltTabDetection = val;
	end,
	get = function(info) return P.attAltTabDetection end
}

---------
CoreOptionsTable.args.core.args.eventHeader = {
	type = "header", order	= 10,
	width	= "full",
	name	= "Events",
}

CoreOptionsTable.args.core.args.ATTIdle = {
	type = "toggle",	order	= 11,
	name = "BG Idle",
	desc = "Gain BG idle debuff",
	set = function(info,val) 
		P.ATTIdle = val;
	end,
	get = function(info) return P.ATTIdle end
}
CoreOptionsTable.args.core.args.ATTDeserter = {
	type = "toggle",	order	= 12,
	name = "Deserter Expire",
	desc = "Deserter debuff expires",
	set = function(info,val) 
		P.ATTDeserter = val;
	end,
	get = function(info) return P.ATTDeserter end
}

CoreOptionsTable.args.core.args.ATTWhisper = {
	type = "toggle",	order	= 13,
	name = "Whisper",
	desc = "Whispers",
	set = function(info,val) 
		P.ATTWhisper = val;
	end,
	get = function(info) return P.ATTWhisper end
}

CoreOptionsTable.args.core.args.ATTCombat = {
	type = "toggle",	order	= 14,
	name = "Combat",
	desc = "Enter combat",
	set = function(info,val) 
		P.ATTCombat = val;
	end,
	get = function(info) return P.ATTCombat end
}

CoreOptionsTable.args.core.args.ATTFlight = {
	type = "toggle",	order	= 15,
	name = "Flight",
	desc = "flight land",
	set = function(info,val) 
		P.ATTFlight = val;
	end,
	get = function(info) return P.ATTFlight end
}

CoreOptionsTable.args.core.args.ATTReadyCheck = {
	type = "toggle",	order	= 16,
	name = "Ready Check",
	desc = "Group Ready Check",
	set = function(info,val) 
		P.ATTReadyCheck = val;
	end,
	get = function(info) return P.ATTReadyCheck end
}
CoreOptionsTable.args.core.args.ATTSummon = {
	type = "toggle",	order	= 17,
	name = "Summon",
	desc = "Summon requests",
	set = function(info,val) 
		P.ATTSummon = val;
	end,
	get = function(info) return P.ATTSummon end
}
CoreOptionsTable.args.core.args.ATTInvite = {
	type = "toggle",	order	= 18,
	name = "Invite",
	desc = "Group invite",
	set = function(info,val) 
		P.ATTInvite = val;
	end,
	get = function(info) return P.ATTInvite end
}
CoreOptionsTable.args.core.args.ATTBGPop = {
	type = "toggle",	order	= 19,
	name = "BG confirm",
	desc = "Battleground confirm to enter",
	set = function(info,val) 
		P.ATTBGPop = val;
	end,
	get = function(info) return P.ATTBGPop end
}
CoreOptionsTable.args.core.args.ATTEnemyDetected = {
	type = "toggle",	order	= 20,
	name = "Enemy Detected",
	desc = "When a Enemy is detected in combatlog. (non-BG/city only)",
	set = function(info,val) 
		P.ATTEnemyDetected = val;
	end,
	get = function(info) return P.ATTEnemyDetected end
}
CoreOptionsTable.args.core.args.ATTBGGatesOpening = {
	type = "toggle",	order	= 21,
	name = "Gates Opening",
	desc = "BG Gates opening",
	set = function(info,val) 
		P.ATTBGGatesOpening = val;
	end,
	get = function(info) return P.ATTBGGatesOpening end
}